@@ -1,7 +1,11 @@
 #' Plot estimated and fitted model parameters
 #'
 #' @param vst_out The output of a vst run
-#' @param show_var Whether to show the average model variance; boolean; default is FALSE
+#' @param show_theta Whether to show the theta parameter; default is FALSE (only the overdispersion factor is shown)
+#' @param show_var Whether to show the average model variance; default is FALSE
+#' @param verbosity An integer specifying whether to show only messages (1), messages and progress bars (2) or nothing (0) while the function is running; default is 2
+#' @param verbose Deprecated; use verbosity instead
+#' @param show_progress Deprecated; use verbosity instead
 #'
 #' @return A ggplot object
 #'
@@ -11,59 +15,114 @@
 #' @export
 #'
 #' @examples
-#' \dontrun{
+#' \donttest{
 #' vst_out <- vst(pbmc, return_gene_attr = TRUE)
 #' plot_model_pars(vst_out)
 #' }
 #'
-plot_model_pars <- function(vst_out, show_var = FALSE) {
+plot_model_pars <- function(vst_out, show_theta = FALSE, show_var = FALSE,
+                            verbosity = 2, verbose = NULL, show_progress = NULL) {
+  # Take care of deprecated arguments
+  if (!is.null(verbose)) {
+    warning("The 'verbose' argument is deprecated as of v0.3. Use 'verbosity' instead. (in sctransform::vst)", immediate. = TRUE, call. = FALSE)
+    verbosity <- as.numeric(verbose)
+  }
+  if (!is.null(show_progress)) {
+    warning("The 'show_progress' argument is deprecated as of v0.3. Use 'verbosity' instead. (in sctransform::vst)", immediate. = TRUE, call. = FALSE)
+    if (show_progress) {
+      verbosity <- 2
+    } else {
+      verbosity <- min(verbosity, 1)
+    }
+  }
+
   if (! 'gmean' %in% names(vst_out$gene_attr)) {
     stop('vst_out must contain a data frame named gene_attr with a column named gmean (perhaps call vst with return_gene_attr = TRUE)')
   }
-  #tmp model pars
-  mp <- vst_out$model_pars
-  mp[, 1] <- log10(mp[, 1])
-  colnames(mp)[1] <- 'log10(theta)'
-  ordered_par_names <- colnames(mp)[c(2:ncol(mp), 1)]
-  if (show_var) {
-    mp <- cbind(mp, log10(get_model_var(vst_out, use_nonreg = TRUE)))
-    colnames(mp)[ncol(mp)] <- 'log10(model var)'
-    ordered_par_names <- c(ordered_par_names, 'log10(model var)')
-  }
-  mp_fit <- vst_out$model_pars_fit
-  mp_fit[, 1] <- log10(mp_fit[, 1])
-  colnames(mp_fit)[1] <- 'log10(theta)'
-  if (show_var) {
-    mp_fit <- cbind(mp_fit, log10(get_model_var(vst_out, use_nonreg = FALSE)))
-    colnames(mp_fit)[ncol(mp_fit)] <- 'log10(model var)'
-  }
+
+  # first handle the per-gene estimates
+  mp <- get_model_par_mat(vst_out, model_pars = vst_out$model_pars, use_nonreg = TRUE,
+                          show_theta = show_theta, show_var = show_var,
+                          verbosity = verbosity)
+  ordered_par_names <- colnames(mp)
+
+  # second the regularized estimates
+  mp_fit <- get_model_par_mat(vst_out, model_pars = vst_out$model_pars_fit, use_nonreg = FALSE,
+                              show_theta = show_theta, show_var = show_var,
+                              verbosity = verbosity)
+
   mpnr <- vst_out$model_pars_nonreg
   if (!is.null(dim(mpnr))) {
     colnames(mpnr) <- paste0('nonreg:', colnames(mpnr))
-    mp_fit <- cbind(mp_fit, mpnr)
+    mp <- cbind(mp, mpnr)
+    ordered_par_names <- c(ordered_par_names, colnames(mpnr))
   }
   # show estimated and regularized parameters
   df <- melt(mp, varnames = c('gene', 'parameter'), as.is = TRUE)
   df$parameter <- factor(df$parameter, levels = ordered_par_names)
   df_fit <- melt(mp_fit, varnames = c('gene', 'parameter'), as.is = TRUE)
   df_fit$parameter <- factor(df_fit$parameter, levels = ordered_par_names)
-  df$gene_gmean <- vst_out$gene_attr[df$gene, 'gmean']
   df$is_outl <- vst_out$model_pars_outliers
-  df_fit$gene_gmean <- vst_out$gene_attr[df_fit$gene, 'gmean']
   df$type <- 'single gene estimate'
   df_fit$type <- 'regularized'
   df_fit$is_outl <- FALSE
+  
+  if (startsWith(x = vst_out$arguments$method, prefix = 'offset')) {
+    df$x <- vst_out$gene_attr[df$gene, 'amean']
+    df_fit$x <- vst_out$gene_attr[df_fit$gene, 'amean']
+    xlab <- 'Arithmetic mean of gene [log10]'
+  } else {
+    df$x <- vst_out$gene_attr[df$gene, 'gmean']
+    df_fit$x <- vst_out$gene_attr[df_fit$gene, 'gmean']
+    xlab <- 'Geometric mean of gene [log10]'
+  }
+  
   df_plot <- rbind(df, df_fit)
   df_plot$parameter <- factor(df_plot$parameter, levels = ordered_par_names)
-  g <- ggplot(df_plot, aes_(x=~log10(gene_gmean), y=~value, color=~type)) +
+  
+  if (!vst_out$arguments$do_regularize || startsWith(x = vst_out$arguments$method, prefix = 'offset')) {
+    df_plot <- df_plot[df_plot$type == 'single gene estimate', ]
+    legend_pos <- 'none'
+  } else {
+    legend_pos <- 'bottom'
+  }
+  
+  g <- ggplot(df_plot, aes_(x=~log10(x), y=~value, color=~type)) +
     geom_point(data=df, aes_(shape=~is_outl), size=0.5, alpha=0.5) +
     scale_shape_manual(values=c(16, 4), guide = FALSE) +
     geom_point(data=df_fit, size=0.66, alpha=0.5, shape=16) +
     facet_wrap(~ parameter, scales = 'free_y', ncol = ncol(mp)) +
-    theme(legend.position='bottom')
+    theme(legend.position = legend_pos) +
+    xlab(label = xlab)
   return(g)
 }
 
+# helper function to plot model parameters
+get_model_par_mat <- function(vst_out, model_pars, use_nonreg, show_theta = FALSE, show_var = FALSE,
+                              verbosity = 2) {
+  mp <- model_pars
+  # transform theta to overdispersion factor
+  if (startsWith(x = vst_out$arguments$method, prefix = 'offset')) {
+    mp[, 1] <- log10(1 + vst_out$gene_attr[rownames(mp), 'amean'] / mp[, 'theta'])
+  } else {
+    mp[, 1] <- log10(1 + vst_out$gene_attr[rownames(mp), 'gmean'] / mp[, 'theta'])
+  }
+  
+  colnames(mp)[1] <- 'log10(od_factor)'
+  ordered_par_names <- colnames(mp)[c(2:ncol(mp), 1)]
+  if (show_theta) {
+    mp <- cbind(mp, log10(model_pars[, 'theta']))
+    colnames(mp)[ncol(mp)] <- 'log10(theta)'
+    ordered_par_names <- c(ordered_par_names, 'log10(theta)')
+  }
+  if (show_var) {
+    mp <- cbind(mp, log10(get_model_var(vst_out, use_nonreg = use_nonreg, verbosity = verbosity)))
+    colnames(mp)[ncol(mp)] <- 'log10(model var)'
+    ordered_par_names <- c(ordered_par_names, 'log10(model var)')
+  }
+  return(mp[, ordered_par_names])
+}
+
 
 # helper function to plot model fit for a single gene
 # returns list with mean, sd, pearson residual
@@ -82,7 +141,7 @@ get_nb_fit <- function(x, umi, gene, cell_attr, as_poisson = FALSE) {
   res <- pmin(res, x$arguments$res_clip_range[2])
   res <- pmax(res, x$arguments$res_clip_range[1])
   ret_df <- data.frame(mu = mu, sd = sd, res = res)
-  # in case we have individaul (non-regularized) parameters
+  # in case we have individual (non-regularized) parameters
   if (gene %in% rownames(x$model_pars)) {
     coefs <- x$model_pars[gene, -1, drop=FALSE]
     theta <- x$model_pars[gene, 1]
@@ -121,19 +180,20 @@ get_nb_fit <- function(x, umi, gene, cell_attr, as_poisson = FALSE) {
 #' @import ggplot2
 #' @import reshape2
 #' @importFrom gridExtra grid.arrange
+#' @importFrom stats bw.nrd0
 #'
 #' @export
 #'
 #' @examples
-#' \dontrun{
+#' \donttest{
 #' vst_out <- vst(pbmc, return_cell_attr = TRUE)
-#' plot_model(vst_out, pbmc, 'PPBP')
+#' plot_model(vst_out, pbmc, 'EMC4')
 #' }
 #'
 plot_model <- function(x, umi, goi, x_var = x$arguments$latent_var[1], cell_attr = x$cell_attr,
                        do_log = TRUE, show_fit = TRUE, show_nr = FALSE, plot_residual = FALSE,
-                       batches = NULL, as_poisson = FALSE, arrange_vertical = TRUE, show_density = TRUE,
-                       gg_cmds = NULL) {
+                       batches = NULL, as_poisson = FALSE, arrange_vertical = TRUE, 
+                       show_density = FALSE, gg_cmds = NULL) {
   if (is.null(batches)) {
     if (!is.null(x$arguments$batch_var)) {
       batches <- cell_attr[, x$arguments$batch_var]
@@ -167,7 +227,9 @@ plot_model <- function(x, umi, goi, x_var = x$arguments$latent_var[1], cell_attr
   df$gene <- factor(df$gene, ordered=TRUE, levels=unique(df$gene))
   g <- ggplot(df, aes_(~x, ~y)) + geom_point(alpha=0.5, shape=16)
   if (show_density) {
-    g <- g + geom_density_2d(color = 'lightblue', size=0.5)
+    bandwidths <- c(bw.nrd0(g$data$x), bw.nrd0(g$data$y))
+    g <- g + geom_density_2d(color = 'lightblue', size=0.5, h = bandwidths,
+                             contour_var = "ndensity")
   }
   if (show_fit) {
     for (b in unique(df$batch)) {
