@@ -1,4 +1,57 @@
 
+# Check cell attributes; add missing ones
+make_cell_attr <- function(umi, cell_attr, latent_var, batch_var, latent_var_nonreg, verbosity) {
+  if (is.null(cell_attr)) {
+    cell_attr <- data.frame(row.names = colnames(umi))
+  }
+  
+  # Do not allow certain variable names
+  no_good <- c('(Intercept)', 'Intercept')
+  if (any(no_good %in% c(latent_var, batch_var, latent_var_nonreg))) {
+    stop('Do not use the following variable names for a latent variable or batch variable: ', paste(no_good, collapse = ', '))
+  }
+
+  # these are the cell attributes that we know how to calculate given the count matrix
+  known_attr <- c('umi', 'gene', 'log_umi', 'log_gene', 'umi_per_gene', 'log_umi_per_gene')
+  # these are the missing cell attributes specified in latent_var
+  missing_attr <- setdiff(c(latent_var, batch_var, latent_var_nonreg), colnames(cell_attr))
+  if (length(missing_attr) > 0) {
+    if (verbosity > 0) {
+      message('Calculating cell attributes from input UMI matrix: ', paste(missing_attr, collapse = ', '))
+    }
+    unknown_attr <- setdiff(missing_attr, known_attr)
+    if (length(unknown_attr) > 0) {
+      stop(sprintf('Unknown cell attributes: %s. Check latent_var, batch_var and latent_var_nonreg and make sure the variables are in cell_attr', paste(unknown_attr, collapse = ', ')))
+    }
+    new_attr <- list()
+    if (any(c('umi', 'log_umi', 'umi_per_gene', 'log_umi_per_gene') %in% missing_attr)) {
+      new_attr$umi <- colSums(umi)
+      new_attr$log_umi <- log10(new_attr$umi)
+    }
+    if (any(c('gene', 'log_gene', 'umi_per_gene', 'log_umi_per_gene') %in% missing_attr)) {
+      new_attr$gene <- colSums(umi > 0)
+      new_attr$log_gene <- log10(new_attr$gene)
+    }
+    if (any(c('umi_per_gene', 'log_umi_per_gene') %in% missing_attr)) {
+      new_attr$umi_per_gene <- new_attr$umi / new_attr$gene
+      new_attr$log_umi_per_gene <- log10(new_attr$umi_per_gene)
+    }
+    new_attr <- do.call(cbind, new_attr)
+    cell_attr <- cbind(cell_attr, new_attr[, setdiff(colnames(new_attr), colnames(cell_attr)), drop = TRUE])
+  }
+  
+  # make sure no NA, NaN, Inf values are in cell attributes - they would cause
+  # problems later on
+  rel_attr <- cell_attr[, c(latent_var, batch_var, latent_var_nonreg)]
+  if (any(is.na(rel_attr)) || 
+      any(is.nan(rel_attr)) || 
+      any(is.infinite(rel_attr))) {
+    stop('cell attributes cannot contain any NA, NaN, or infinite values')
+  }
+  
+  return(cell_attr)
+}
+
 #' Geometric mean per row
 #'
 #' @param x matrix of class \code{matrix} or \code{dgCMatrix}
@@ -10,24 +63,25 @@ row_gmean <- function(x, eps = 1) {
     return(exp(rowMeans(log(x + eps))) - eps)
   }
   if (inherits(x = x, what = 'dgCMatrix')) {
-    ret <- row_gmean_dgcmatrix(x = x@x, i = x@i, rows = nrow(x), cols = ncol(x), eps = eps)
+    ret <- row_gmean_dgcmatrix(matrix = x, eps = eps)
     names(ret) <- rownames(x)
     return(ret)
   }
   stop('matrix x needs to be of class matrix or dgCMatrix')
 }
 
+
+
 #' Variance per row
 #'
 #' @param x matrix of class \code{matrix} or \code{dgCMatrix}
 #'
 #' @return variances
+#' 
+#' @importFrom matrixStats rowVars
 row_var <- function(x) {
   if (inherits(x = x, what = 'matrix')) {
-    ret <- switch(storage.mode(x),
-                  'double' = row_var_dense_d(x),
-                  'integer' = row_var_dense_i(x),
-                  stop('Unknown matrix storage mode'))
+    ret <- rowVars(x)
     names(ret) <- rownames(x)
     return(ret)
   }
@@ -121,15 +175,17 @@ deviance_residual <- function(y, mu, theta, wt=1) {
 #' @param min_variance Lower bound for the estimated variance for any gene in any cell when calculating pearson residual; default is vst_out$arguments$min_variance
 #' @param cell_attr Data frame of cell meta data
 #' @param bin_size Number of genes to put in each bin (to show progress)
-#' @param show_progress Whether to print progress bar
+#' @param verbosity An integer specifying whether to show only messages (1), messages and progress bars (2) or nothing (0) while the function is running; default is 2
+#' @param verbose Deprecated; use verbosity instead
+#' @param show_progress Deprecated; use verbosity instead
 #'
 #' @return A matrix of residuals
 #'
 #' @export
 #'
 #' @examples
-#' \dontrun{
-#' vst_out <- vst(pbmc)
+#' \donttest{
+#' vst_out <- vst(pbmc, return_cell_attr = TRUE)
 #' pearson_res <- get_residuals(vst_out, pbmc)
 #' deviance_res <- get_residuals(vst_out, pbmc, residual_type = 'deviance')
 #' }
@@ -137,7 +193,22 @@ deviance_residual <- function(y, mu, theta, wt=1) {
 get_residuals <- function(vst_out, umi, residual_type = 'pearson',
                           res_clip_range = c(-sqrt(ncol(umi)), sqrt(ncol(umi))),
                           min_variance = vst_out$arguments$min_variance,
-                          cell_attr = vst_out$cell_attr, bin_size = 256, show_progress = TRUE) {
+                          cell_attr = vst_out$cell_attr, bin_size = 256,
+                          verbosity = vst_out$arguments$verbosity, 
+                          verbose = NULL, show_progress = NULL) {
+  # Take care of deprecated arguments
+  if (!is.null(verbose)) {
+    warning("The 'verbose' argument is deprecated as of v0.3. Use 'verbosity' instead. (in sctransform::vst)", immediate. = TRUE, call. = FALSE)
+    verbosity <- as.numeric(verbose)
+  }
+  if (!is.null(show_progress)) {
+    warning("The 'show_progress' argument is deprecated as of v0.3. Use 'verbosity' instead. (in sctransform::vst)", immediate. = TRUE, call. = FALSE)
+    if (show_progress) {
+      verbosity <- 2
+    } else {
+      verbosity <- min(verbosity, 1)
+    }
+  }
   regressor_data <- model.matrix(as.formula(gsub('^y', '', vst_out$model_str)), cell_attr)
   model_pars <- vst_out$model_pars_fit
   if (!is.null(dim(vst_out$model_pars_nonreg))) {
@@ -147,12 +218,12 @@ get_residuals <- function(vst_out, umi, residual_type = 'pearson',
   }
 
   genes <- rownames(umi)[rownames(umi) %in% rownames(model_pars)]
-  if (show_progress) {
+  if (verbosity > 0) {
     message('Calculating residuals of type ', residual_type, ' for ', length(genes), ' genes')
   }
   bin_ind <- ceiling(x = 1:length(x = genes) / bin_size)
   max_bin <- max(bin_ind)
-  if (show_progress) {
+  if (verbosity > 1) {
     pb <- txtProgressBar(min = 0, max = max_bin, style = 3)
   }
   res <- matrix(NA_real_, length(genes), nrow(regressor_data), dimnames = list(genes, rownames(regressor_data)))
@@ -162,13 +233,14 @@ get_residuals <- function(vst_out, umi, residual_type = 'pearson',
     y <- as.matrix(umi[genes_bin, , drop=FALSE])
     res[genes_bin, ] <- switch(residual_type,
       'pearson' = pearson_residual(y, mu, model_pars[genes_bin, 'theta'], min_var = min_variance),
-      'deviance' = deviance_residual(y, mu, model_pars[genes_bin, 'theta'])
+      'deviance' = deviance_residual(y, mu, model_pars[genes_bin, 'theta']),
+      stop('residual_type ', residual_type, ' unknown - only pearson and deviance supported at the moment')
     )
-    if (show_progress) {
+    if (verbosity > 1) {
       setTxtProgressBar(pb, i)
     }
   }
-  if (show_progress) {
+  if (verbosity > 1) {
     close(pb)
   }
   res[res < res_clip_range[1]] <- res_clip_range[1]
@@ -187,22 +259,40 @@ get_residuals <- function(vst_out, umi, residual_type = 'pearson',
 #' @param min_variance Lower bound for the estimated variance for any gene in any cell when calculating pearson residual; default is vst_out$arguments$min_variance
 #' @param cell_attr Data frame of cell meta data
 #' @param bin_size Number of genes to put in each bin (to show progress)
-#' @param show_progress Whether to print progress bar
+#' @param verbosity An integer specifying whether to show only messages (1), messages and progress bars (2) or nothing (0) while the function is running; default is 2
+#' @param verbose Deprecated; use verbosity instead
+#' @param show_progress Deprecated; use verbosity instead
 #'
 #' @return A vector of residual variances (after clipping)
 #'
 #' @export
 #'
 #' @examples
-#' \dontrun{
-#' vst_out <- vst(pbmc)
+#' \donttest{
+#' vst_out <- vst(pbmc, return_cell_attr = TRUE)
 #' res_var <- get_residual_var(vst_out, pbmc)
 #' }
 #'
 get_residual_var <- function(vst_out, umi, residual_type = 'pearson',
                              res_clip_range = c(-sqrt(ncol(umi)), sqrt(ncol(umi))),
                              min_variance = vst_out$arguments$min_variance,
-                             cell_attr = vst_out$cell_attr, bin_size = 256, show_progress = TRUE) {
+                             cell_attr = vst_out$cell_attr, bin_size = 256,
+                             verbosity = vst_out$arguments$verbosity, 
+                             verbose = NULL, show_progress = NULL) {
+  # Take care of deprecated arguments
+  if (!is.null(verbose)) {
+    warning("The 'verbose' argument is deprecated as of v0.3. Use 'verbosity' instead. (in sctransform::vst)", immediate. = TRUE, call. = FALSE)
+    verbosity <- as.numeric(verbose)
+  }
+  if (!is.null(show_progress)) {
+    warning("The 'show_progress' argument is deprecated as of v0.3. Use 'verbosity' instead. (in sctransform::vst)", immediate. = TRUE, call. = FALSE)
+    if (show_progress) {
+      verbosity <- 2
+    } else {
+      verbosity <- min(verbosity, 1)
+    }
+  }
+
   regressor_data <- model.matrix(as.formula(gsub('^y', '', vst_out$model_str)), cell_attr)
   model_pars <- vst_out$model_pars_fit
   if (!is.null(dim(vst_out$model_pars_nonreg))) {
@@ -212,12 +302,12 @@ get_residual_var <- function(vst_out, umi, residual_type = 'pearson',
   }
 
   genes <- rownames(umi)[rownames(umi) %in% rownames(model_pars)]
-  if (show_progress) {
+  if (verbosity > 0) {
     message('Calculating variance for residuals of type ', residual_type, ' for ', length(genes), ' genes')
   }
   bin_ind <- ceiling(x = 1:length(x = genes) / bin_size)
   max_bin <- max(bin_ind)
-  if (show_progress) {
+  if (verbosity > 1) {
     pb <- txtProgressBar(min = 0, max = max_bin, style = 3)
   }
   res <- matrix(NA_real_, length(genes))
@@ -228,15 +318,16 @@ get_residual_var <- function(vst_out, umi, residual_type = 'pearson',
     y <- as.matrix(umi[genes_bin, , drop=FALSE])
     res_mat <- switch(residual_type,
                       'pearson' = pearson_residual(y, mu, model_pars[genes_bin, 'theta'], min_var = min_variance),
-                      'deviance' = deviance_residual(y, mu, model_pars[genes_bin, 'theta']))
+                      'deviance' = deviance_residual(y, mu, model_pars[genes_bin, 'theta']),
+                      stop('residual_type ', residual_type, ' unknown - only pearson and deviance supported at the moment'))
     res_mat[res_mat < res_clip_range[1]] <- res_clip_range[1]
     res_mat[res_mat > res_clip_range[2]] <- res_clip_range[2]
     res[genes_bin] <- row_var(res_mat)
-    if (show_progress) {
+    if (verbosity > 1) {
       setTxtProgressBar(pb, i)
     }
   }
-  if (show_progress) {
+  if (verbosity > 1) {
     close(pb)
   }
   return(res)
@@ -250,19 +341,37 @@ get_residual_var <- function(vst_out, umi, residual_type = 'pearson',
 #' @param cell_attr Data frame of cell meta data
 #' @param use_nonreg Use the non-regularized parameter estimates; boolean; default is FALSE
 #' @param bin_size Number of genes to put in each bin (to show progress)
-#' @param show_progress Whether to print progress bar
+#' @param verbosity An integer specifying whether to show only messages (1), messages and progress bars (2) or nothing (0) while the function is running; default is 2
+#' @param verbose Deprecated; use verbosity instead
+#' @param show_progress Deprecated; use verbosity instead
 #'
 #' @return A named vector of variances (the average across all cells), one entry per gene.
 #'
 #' @export
 #'
 #' @examples
-#' \dontrun{
-#' vst_out <- vst(pbmc)
+#' \donttest{
+#' vst_out <- vst(pbmc, return_cell_attr = TRUE)
 #' res_var <- get_model_var(vst_out)
 #' }
 #'
-get_model_var <- function(vst_out, cell_attr = vst_out$cell_attr, use_nonreg = FALSE, bin_size = 256, show_progress = TRUE) {
+get_model_var <- function(vst_out, cell_attr = vst_out$cell_attr, use_nonreg = FALSE,
+                          bin_size = 256, verbosity = 2,
+                          verbose = NULL, show_progress = NULL) {
+  # Take care of deprecated arguments
+  if (!is.null(verbose)) {
+    warning("The 'verbose' argument is deprecated as of v0.3. Use 'verbosity' instead. (in sctransform::vst)", immediate. = TRUE, call. = FALSE)
+    verbosity <- as.numeric(verbose)
+  }
+  if (!is.null(show_progress)) {
+    warning("The 'show_progress' argument is deprecated as of v0.3. Use 'verbosity' instead. (in sctransform::vst)", immediate. = TRUE, call. = FALSE)
+    if (show_progress) {
+      verbosity <- 2
+    } else {
+      verbosity <- min(verbosity, 1)
+    }
+  }
+
   regressor_data <- model.matrix(as.formula(gsub('^y', '', vst_out$model_str)), cell_attr)
   if (use_nonreg) {
     model_pars <- vst_out$model_pars
@@ -276,12 +385,12 @@ get_model_var <- function(vst_out, cell_attr = vst_out$cell_attr, use_nonreg = F
   }
 
   genes <- rownames(model_pars)
-  if (show_progress) {
+  if (verbosity > 0) {
     message('Calculating model variance for ', length(genes), ' genes')
   }
   bin_ind <- ceiling(x = 1:length(x = genes) / bin_size)
   max_bin <- max(bin_ind)
-  if (show_progress) {
+  if (verbosity > 1) {
     pb <- txtProgressBar(min = 0, max = max_bin, style = 3)
   }
   res <- matrix(NA_real_, length(genes))
@@ -291,11 +400,11 @@ get_model_var <- function(vst_out, cell_attr = vst_out$cell_attr, use_nonreg = F
     mu <- exp(tcrossprod(model_pars[genes_bin, -1, drop=FALSE], regressor_data))
     model_var = mu + mu^2 / model_pars[genes_bin, 'theta']
     res[genes_bin] <- rowMeans(model_var)
-    if (show_progress) {
+    if (verbosity > 1) {
       setTxtProgressBar(pb, i)
     }
   }
-  if (show_progress) {
+  if (verbosity > 1) {
     close(pb)
   }
   return(res)
